#include "StdAfx.h"
#include ".\[!output SAFE_PROJECT_NAME]Exporter.h"
[!if SETTINGS_DIALOG]
#include ".\settingsdialog.h"
[!endif]
[!if THUMBNAILS]
#include <atlimage.h>
[!endif]

//*******************************************************************************
// Name : CBounds
// Desc : Constructor
//*******************************************************************************
CBounds::CBounds(double x1, double y1, double x2, double y2)
: xMin(x1), yMin(y1), xMax(x2), yMax(y2)
{
}



std::wstring C[!output SAFE_PROJECT_NAME]Exporter::g_strErr;


//*******************************************************************************
// Name : C[!output SAFE_PROJECT_NAME]Exporter
// Desc : Default constructor
//*******************************************************************************
C[!output SAFE_PROJECT_NAME]Exporter::C[!output SAFE_PROJECT_NAME]Exporter(BatchExportBeginInfo *pInfo)
: m_bounds(pInfo->xMin, pInfo->yMin, pInfo->xMax, pInfo->yMax)
, m_nNumberOfThemesInBatch(pInfo->nThemeCount)
, m_clrBkgnd(pInfo->clrBkgnd)
{
	// TODO: Set pInfo->bExportLayerZOrderBottomToTop
	//       = true, the layer in the bottom z-order will be exorted first
	//       = false, the layer in the top z-order will be exported first
}


//*******************************************************************************
// Name : ~C[!output SAFE_PROJECT_NAME]Exporter
// Desc : Destructor
//*******************************************************************************
C[!output SAFE_PROJECT_NAME]Exporter::~C[!output SAFE_PROJECT_NAME]Exporter(void)
{
}


//*******************************************************************************
// Name : GetName
// Desc : Return the name that will be displayed in ShapeUp menues.
//*******************************************************************************
const wchar_t* C[!output SAFE_PROJECT_NAME]Exporter::GetName()
{
	return L"Friendly name of [!output PROJECT_NAME]";
}


//*******************************************************************************
// Name : GetDescription
// Desc : Return a one-line description of this exporter to be used in ShapeUp.
//*******************************************************************************
const wchar_t* C[!output SAFE_PROJECT_NAME]Exporter::GetDescription()
{
	return L"Exports some data";
}


//*******************************************************************************
// Name : GetLastErrMsg
// Desc : Return last error message
//*******************************************************************************
const wchar_t* C[!output SAFE_PROJECT_NAME]Exporter::GetLastErrMsg()
{
	return g_strErr.c_str();
}


//*******************************************************************************
// Name : GetInfo
// Desc : Return a description of this Exporter. This is accessible from
//        Tools | Options | Plugins | Exporters | Info button.
//*******************************************************************************
const wchar_t* C[!output SAFE_PROJECT_NAME]Exporter::GetInfo()
{
	return	L"Exporter plugin [!output PROJECT_NAME] version 1.0.\n"
			L"Copyright 2006.";
}


//*******************************************************************************
// Name : SupportsType
// Desc : Return the types of layers supported
//*******************************************************************************
bool C[!output SAFE_PROJECT_NAME]Exporter::SupportsType(ShapeType t)
{
[!if LOAD_POINT]
	if (t == Shape_Point)
		return true;

[!endif]
[!if LOAD_POLYLINE]
	if (t == Shape_Arc)
		return true;

[!endif]
[!if LOAD_POLYGON]
	if (t == Shape_Polygon)
		return true;

[!endif]
	// We do not support this type
	return false;
}


//*******************************************************************************
// Name : OnPreExportLayer
// Desc : This function gets called once at the start of this session.
//*******************************************************************************
bool C[!output SAFE_PROJECT_NAME]Exporter::OnPreExportLayer(PreExportInfo *pInfo)
{
	// TODO: This is a good place to prepare date storage.
	//
	//       If you want the user to make any per layer exporting selections you
	//       can display a dialog box from here. E.g. if loading data from a
	//       file, a file dialog could be placed here.

[!if FILE_SUPPORT]
	{
		// BLOCK: File support
		// TODO: This file dialog will open for every layer exported. Thus, exporting
		//       each layer to a different file.
		//       If you want to export all layers to a single file, move this block
		//       to the constructor. In that case, you cannot simply return if the
		//       user presses cancel. You will need to set an interrupt state variable,
		//       e.g. member bool m_bCancel = true, and if this is set, you should
		//       return false from this function.
		char szFilter[] = "MyType Files (*.my)|*.my|All Files (*.*)|*.*||";
		CFileDialog dlg(FALSE, 0, 0, OFN_HIDEREADONLY | OFN_EXPLORER, szFilter);
		if (dlg.DoModal() != IDOK)
		{
			// User pressed cancel
			return false;
		}

		m_strFileName = dlg.GetPathName();
	}

[!endif]
	// Returning false will cancel the export of this layer
	return true;
}


//*******************************************************************************
// Name : OnPostExportLayer
// Desc : Called after each layer has been exported
//*******************************************************************************
bool C[!output SAFE_PROJECT_NAME]Exporter::OnPostExportLayer(ShapeType type)
{
	// TODO: Handle any export efter each layer
	return true;
}


[!if SETTINGS_DIALOG]
//*******************************************************************************
// Name : OnSettings
// Desc : 
//*******************************************************************************
void C[!output SAFE_PROJECT_NAME]Exporter::OnSettings()
{
	CSettingsDialog dlg;

	// TODO: Populate dlg with data from registry

	if (dlg.DoModal())
	{
		// TODO: Collect data from dlg and store in registry
	}
}


[!endif]
//*******************************************************************************
// Name : OnExportShape
// Desc : 
//*******************************************************************************
bool C[!output SAFE_PROJECT_NAME]Exporter::OnExportShape(ExportShapeInfo *pInfo)
{
	// TODO: Extract shape data and store it.

	switch (pInfo->type)
	{
[!if LOAD_POINT]
	case Shape_Point:
		{
			CShapePointRecord *pShape = reinterpret_cast<CShapePointRecord*>(pInfo->pShape);

			double x = pShape->GetX();
			double y = pShape->GetY();
			// TODO: Store point data
[!if SUPPORTS_ATTRIBUTES]

			// Get attribute data
			for (size_t nCol = 0; nCol < pInfo->nAttributeCount; ++nCol)
			{
				// IMPORTANT: The callback returns a pointer to a const char. This
				//            could point to a static buffer, which may change
				//            between successive calls. Make sure data is handled
				//            (copied or stored) before the next call.
				std::string strAttr(pInfo->pfnACB(pShape->GetIndex(), nCol));
				// TODO: store attribute data
			}
[!endif]
		}
		break;
[!endif]
[!if LOAD_POLYLINE]
	case Shape_Arc:
		{
			CShapeArcRecord *pShape = reinterpret_cast<CShapeArcRecord*>(pInfo->pShape);

			size_t nPartCount = pShape->GetPartCount();

			// Loop through each part
			for (size_t nPart = 0; nPart < nPartCount; nPart++)
			{
				size_t begin = pShape->GetPart(nPart);
				size_t end = (nPart == (nPartCount - 1)) ? pShape->GetPointCount() : pShape->GetPart(nPart + 1);

				// size_t pointCount = end - begin;

				// Loop through each point in this part
				for (size_t nPoint = begin; nPoint < end; nPoint++)
				{
					double x = pShape->GetPoint(nPoint).x;
					double y = pShape->GetPoint(nPoint).y;
					// TODO: Store point data
				}
			}
[!if SUPPORTS_ATTRIBUTES]

			// Get attribute data
			for (size_t nCol = 0; nCol < pInfo->nAttributeCount; ++nCol)
			{
				// IMPORTANT: The callback returns a pointer to a const char. This
				//            could point to a static buffer, which may change
				//            between successive calls. Make sure data is handled
				//            (copied or stored) before the next call.
				std::string strAttr(pInfo->pfnACB(pShape->GetIndex(), nCol));
				// TODO: store attribute data
			}
[!endif]
		}
		break;
[!endif]
[!if LOAD_POLYGON]
	case Shape_Polygon:
		{
			CShapePolygonRecord *pShape = reinterpret_cast<CShapePolygonRecord*>(pInfo->pShape);

			size_t nPartCount = pShape->GetPartCount();

			// Loop through each part
			for (size_t nPart = 0; nPart < nPartCount; nPart++)
			{
				size_t begin = pShape->GetPart(nPart);
				size_t end = (nPart == (nPartCount - 1)) ? pShape->GetPointCount() : pShape->GetPart(nPart + 1);

				// size_t pointCount = end - begin;

				// Loop through each point in this part
				for (size_t nPoint = begin; nPoint < end; nPoint++)
				{
					double x = pShape->GetPoint(nPoint).x;
					double y = pShape->GetPoint(nPoint).y;
					// TODO: Store point data
				}
			}
[!if SUPPORTS_ATTRIBUTES]

			// Get attribute data
			for (size_t nCol = 0; nCol < pInfo->nAttributeCount; ++nCol)
			{
				// IMPORTANT: The callback returns a pointer to a const char. This
				//            could point to a static buffer, which may change
				//            between successive calls. Make sure data is handled
				//            (copied or stored) before the next call.
				std::string strAttr(pInfo->pfnACB(pShape->GetIndex(), nCol));
				// TODO: store attribute data
			}
[!endif]
		}
		break;
[!endif]
	default:
		// Ending up here implies a coding error in SupportsType()
		ASSERT(FALSE);
		return false;
	}
[!if THUMBNAILS]

	{
		// BLOCK: Generate a map thumb nail image
		MapImageInfo mii;
		::memset(&mii, 0, sizeof(MapImageInfo));

		// Setup wanted image size in pixels
		mii.imgWidth = 51;
		mii.imgHeight = 51;

		// Setup wanted center pos
		mii.centerGeoPosX = pInfo->pShape->GetXMin();
		mii.centerGeoPosY = pInfo->pShape->GetYMin();

		// Use the current map view scale 
		mii.geoWidth = 0.;
		mii.geoHeight = 0.;

		// Get out thumb nail
		HBITMAP hbm = pInfo->pfnMICB(&mii);

		if (hbm != NULL)
		{
			// Success
			CImage img;
			img.Attach(hbm);

			// Do something with the image
			img.Save("C:\\thumbnail.png", Gdiplus::ImageFormatPNG);

			// We are responsible for destroying the bitmap, hbm,
			// but since it was attached to a CImage object,
			// that object will destroy the bitmap for us.
		}
	}
[!endif]

	// Return false if this shape could not be exported
	return true;
}

