#ifndef _XPRDEF_H
#define _XPRDEF_H

/***************************************************************************
 *
 * Definition file for ShapeUp Exporter plugins.
 *
 ***************************************************************************/

#include "./shapeobjects.h"
#include <tchar.h>

#define EXPORTER_PLUGIN_INTERFACE_VERSION 5

// Attribute types
//   Any of the types kan be combined with the AT_PRIMARYKEY flag
//   to indicate that the attribute is part of a primary key.
#define AT_STRING     0
#define AT_INT        1
#define AT_DOUBLE     2
#define AT_DATE       3
#define AT_BOOL       4
#define AT_PRIMARYKEY 0x80000000

typedef struct tagMapImageInfo
{
	int imgWidth;
	int imgHeight;
	double centerGeoPosX;
	double centerGeoPosY;
	// Geo width and height are optional. If set to 0.0, the current values in
	// ShapeUp map view will be used.
	double geoWidth;
	double geoHeight;
} MapImageInfo;

typedef const char*   (__cdecl *PFNATTRIBUTENAMECALLBACK)(size_t nCol);
typedef unsigned long (__cdecl *PFNATTRIBUTETYPECALLBACK)(size_t nCol);
typedef const char*   (__cdecl *PFNATTRIBUTECALLBACK)(int nShapeIndex, size_t nCol);
typedef HBITMAP       (__cdecl *PFNMAPIMAGECALLBACK)(const MapImageInfo *pInfo);

/***************************************************************************
 *
 * The following section contains functions that must be implemented.
 *
 ***************************************************************************/

const wchar_t* __cdecl
	fnGetName
		(
			void
		);

const wchar_t* __cdecl
	fnGetDescription
		(
			void
		);

const wchar_t* __cdecl
	fnGetLastErrMsg
		(
			void
		);

typedef struct tagExportShapeInfo
{
	ShapeType type;
	CShapeRecord* pShape;
	size_t nAttributeCount;
	COLORREF clrPen;
	size_t nPenWidth;
	int nIconIndex;
	PFNATTRIBUTENAMECALLBACK pfnANCB;
	PFNATTRIBUTECALLBACK pfnACB;
	PFNATTRIBUTETYPECALLBACK pfnATCB;
	PFNMAPIMAGECALLBACK pfnMICB;
	int nPenStyle;
	COLORREF clrFill;
	bool bIsSelected;
	const char *pszLabel;
} ExportShapeInfo;

bool __cdecl
	fnExportShape
		(
			ExportShapeInfo *pInfo
		);

bool __cdecl
	fnSupportsType
		(
			ShapeType t,
			size_t nAttributeCount,
			PFNATTRIBUTENAMECALLBACK pfnANCB
		);

size_t __cdecl
	fnVersion
		(
			void // Current version is EXPORTER_PLUGIN_INTERFACE_VERSION
		);

/***************************************************************************
 *
 * The following section contains functions that are optionally implemented.
 *
 ***************************************************************************/

typedef struct tagPreExportInfo
{
	size_t nCurrentLayerIndex;
	const char* pszCurrentLayerName;
	ShapeType type;
	size_t nShapeCount;
	size_t nAttributeCount;
	PFNATTRIBUTENAMECALLBACK pfnANCB;
	PFNATTRIBUTETYPECALLBACK pfnATCB;
} PreExportInfo;

bool __cdecl
	fnPreExport
		(
			PreExportInfo *pInfo
		);

bool __cdecl
	fnPostExport
		(
			ShapeType type,
			bool bRetCode
		);

typedef struct tagBatchExportBeginInfo
{
/* In */
	size_t   nThemeCount;
	double   xMin;
	double   yMin;
	double   xMax;
	double   yMax;
	COLORREF clrBkgnd;
/* In/Out */
	bool     bExportLayerZOrderBottomToTop;
} BatchExportBeginInfo;

void __cdecl
	fnBatchExportBegin
		(
			BatchExportBeginInfo *pInfo
		);

void __cdecl
	fnBatchExportEnd
		(
			void
		);

const wchar_t* __cdecl
	fnGetInfo
		(
			void
		);

void __cdecl
	fnSettings
		(
			void
		);


#endif /* _XPRDEF_H */
