// ShapePolygonCreator.cpp: implementation of the CShapePolygonCreator class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "ShapePolygonCreator.h"
#include "shapeobjects.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

//*******************************************************************************
// Name : CShapePolygonCreator
// Desc : 
//*******************************************************************************
CShapePolygonCreator::CShapePolygonCreator(size_t nParts,
										   size_t nTotalPointCount,
										   size_t nRecordNumber)
{
	m_nParts = nParts;
	m_nTotalPointCount = nTotalPointCount;
	m_nRecordNumber = nRecordNumber + 1; // 1-based index
	m_pMem = NULL;
	m_pNextPoint = NULL;
	m_bPointWritten = false;
	m_nCurrentPart = 0;
}


//*******************************************************************************
// Name : ~CShapePolygonCreator
// Desc : 
//*******************************************************************************
CShapePolygonCreator::~CShapePolygonCreator()
{

}


//*******************************************************************************
// Name : SetRecordNumber
// Desc : 
//*******************************************************************************
void CShapePolygonCreator::SetRecordNumber(size_t nRecordNumber)
{
	m_nRecordNumber = nRecordNumber + 1;
}


//*******************************************************************************
// Name : SetNewRecordNumber
// Desc :
//*******************************************************************************
void CShapePolygonCreator::SetNewRecordNumber(void* pMem, size_t nRecordNumber)
{
	SShapePolygonRecord *s = reinterpret_cast<SShapePolygonRecord*>(pMem);
	s->m_recordNumber = ByteSwitchInt(nRecordNumber + 1);
}


//*******************************************************************************
// Name : UpdateCoordinate
// Desc : 
//*******************************************************************************
void CShapePolygonCreator::UpdateCoordinate(void* pMem, size_t nIndex, double x, double y)
{
	CShapePolygonRecord *p = reinterpret_cast<CShapePolygonRecord*>(pMem);
	CShapePolygonRecord::XYPair& point = const_cast<CShapePolygonRecord::XYPair&>(p->GetPoint(nIndex));
	point.x = x;
	point.y = y;

	// Last point needs to be the same as first point. We might have to update both.
	if (nIndex == 0)
	{
		CShapePolygonRecord::XYPair& point = const_cast<CShapePolygonRecord::XYPair&>(p->GetPoint(p->GetPointCount() - 1));
		point.x = x;
		point.y = y;
	}
	else if (nIndex == static_cast<size_t>(p->GetPointCount() - 1))
	{
		CShapePolygonRecord::XYPair& point = const_cast<CShapePolygonRecord::XYPair&>(p->GetPoint(0));
		point.x = x;
		point.y = y;
	}

	// Update bounding box
	RecalcBoundingBox(pMem);
}


//*******************************************************************************
// Name : RecalcBoundingBox
// Desc : 
//*******************************************************************************
void CShapePolygonCreator::RecalcBoundingBox(void* pMem)
{
	SShapePolygonRecord *s = reinterpret_cast<SShapePolygonRecord*>(pMem);
	CShapePolygonRecord *p = reinterpret_cast<CShapePolygonRecord*>(pMem);

	if (p->GetPointCount() > 0)
	{
		s->m_xMax = s->m_xMin = p->GetPoint(0).x;
		s->m_yMax = s->m_yMin = p->GetPoint(0).y;
	}
	else
		return;

	for (size_t i = 1; i < p->GetPointCount(); i++)
	{
		double x = p->GetPoint(i).x;
		double y = p->GetPoint(i).y;
		if (x < s->m_xMin)
			s->m_xMin = x;
		if (x > s->m_xMax)
			s->m_xMax = x;
		if (y < s->m_yMin)
			s->m_yMin = y;
		if (y > s->m_yMax)
			s->m_yMax = y;
	}
}


//*******************************************************************************
// Name : GetAllocSizeBytes
// Desc : 
//*******************************************************************************
size_t CShapePolygonCreator::GetAllocSizeBytes() const
{
	return sizeof(SShapePolygonRecord) +
			(m_nParts-1)*sizeof(int) + // -1, one int is already accounted for in SShapePolygonRecord
			m_nTotalPointCount*sizeof(double)*2;
}


//*******************************************************************************
// Name : WriteHeader
// Desc : 
//*******************************************************************************
bool CShapePolygonCreator::WriteHeader(void* pMem)
{
	m_pMem = reinterpret_cast<unsigned char*>(pMem);
	if (m_pMem != NULL)
	{
		SShapePolygonRecord *s = reinterpret_cast<SShapePolygonRecord*>(m_pMem);
		s->m_recordNumber = ByteSwitchInt(m_nRecordNumber);
		s->m_contentLength = ByteSwitchInt(GetAllocSizeBytes() - sizeof(int)*2); // Remove record header length
		s->IntShapeType = Shape_Polygon;
		s->m_nPartCount = m_nParts;
		s->m_nPointCount = m_nTotalPointCount;
		s->m_parts[0] = 0;
		m_nCurrentPart = 0;
		m_bPointWritten = false;
		m_nSuccessfullyWrittenPoints = 0;
		m_pNextPoint = reinterpret_cast<unsigned char*>(&s->m_parts + m_nParts);

		return true;
	}
	return false;
}


//*******************************************************************************
// Name : WritePoint
// Desc : 
//*******************************************************************************
bool CShapePolygonCreator::WritePoint(double x, double y)
{
	if (m_pMem != NULL && m_pNextPoint != NULL)
	{
		if (m_pNextPoint - m_pMem + sizeof(double)*2 > GetAllocSizeBytes())
			return false;

		m_nSuccessfullyWrittenPoints++;

		SShapePolygonRecord *s = reinterpret_cast<SShapePolygonRecord*>(m_pMem);
		if (m_bPointWritten)
		{
			if (x < s->m_xMin)
				s->m_xMin = x;
			if (x > s->m_xMax)
				s->m_xMax = x;
			if (y < s->m_yMin)
				s->m_yMin = y;
			if (y > s->m_yMax)
				s->m_yMax = y;
		}
		else
		{
			s->m_xMin = s->m_xMax = x;
			s->m_yMin = s->m_yMax = y;
			m_bPointWritten = true;
		}
		::memcpy(m_pNextPoint, &x, sizeof(double));
		m_pNextPoint += sizeof(double);
		::memcpy(m_pNextPoint, &y, sizeof(double));
		m_pNextPoint += sizeof(double);

		return true;
	}
	return false;
}


//*******************************************************************************
// Name : WritePoint
// Desc : 
//*******************************************************************************
bool CShapePolygonCreator::WriteNewPart()
{
	if (m_pMem == NULL)
		return false;

	m_nCurrentPart++;

	if (m_nCurrentPart >= m_nParts)
		return false;

	SShapePolygonRecord *s = reinterpret_cast<SShapePolygonRecord*>(m_pMem);
	s->m_parts[m_nCurrentPart] = m_nSuccessfullyWrittenPoints;

	return true;
}

