#ifndef _LDRDEF_H
#define _LDRDEF_H

#include "./shapeobjects.h"
#include <tchar.h>

#define LOADER_PLUGIN_INTERFACE_VERSION 5

/***************************************************************************
 *
 * The following section contains definitions for icon indecies.
 *
 ***************************************************************************/
#define SI_DOT_R3                0
#define SI_DOT_R3_BOUNDLESS      1
#define SI_SQUARE_S5             2
#define SI_SQUARE_S7             3
#define SI_SQUARE_S5_BOUNDLESS   4
#define SI_DIAMOND_S4            5
#define SI_EXCLAMATION_TRIANGLE  6
#define SI_FLASH                 7
#define SI_PIN_SW                8
#define SI_PIN_S                 9
#define SI_POLYGON_5            10
#define SI_HOUSE                11
#define SI_FLAG_LEFT            12
#define SI_FLAG_RIGHT           13
#define SI_2TRIANGLES           14
#define SI_ANTENNA              15
#define SI_ARROW_THIN_NW        16
#define SI_PLUS_S7              17
#define SI_CROSS_S7             18
#define SI_SNOWFLAKE            19
#define SI_LINE_NW              20
#define SI_EMPTY                21


/***************************************************************************
 *
 * The following section contains definitions of callback functions.
 *
 ***************************************************************************/

typedef bool (__cdecl *PFNADDGRAPHICCALLBACK)(unsigned long ulSessionData,
											  CShapeRecord* pShape,
											  size_t nExtraDataSizeBytes);

typedef bool (__cdecl *PFNREMOVEGRAPHICCALLBACK)(unsigned long ulSessionData,
												 CShapeRecord* pShape);

typedef bool (__cdecl *PFNSELECTGRAPHICCALLBACK)(unsigned long ulSessionData,
												 unsigned long ulThemeKey,
												 CShapeRecord* pShape,
												 unsigned long bSelect);

typedef bool (__cdecl *PFNRAISEEVENTCALLBACK)(unsigned long ulSessionData,
											  unsigned long ulUpdateHint);

typedef bool (__cdecl *PFNGETSELECTEDSHAPECALLBACK)(unsigned long ulSessionData,
													unsigned long nIndex,
													CShapeRecord** ppShape);

/***************************************************************************
 *
 * The following section contains functions that must be implemented.
 *
 ***************************************************************************/

const wchar_t* __cdecl
	fnGetName // Plugin name displayed in menu
		(
			void
		);

const wchar_t* __cdecl
	fnGetLastErrMsg
		(
			void
		);

typedef struct tagLoadShapeInfo
{
	ShapeType type;					// in
	double dLongMin;				// in
	double dLatMin;					// in
	double dLongMax;				// in
	double dLatMax;					// in
	PFNADDGRAPHICCALLBACK pfnAGCB;	// in
	unsigned long ulSessionData;	// in, this parameter should go into (*pfnAGCB)()
	size_t nThemeIndex;				// in
	COLORREF clr;					// out
	wchar_t *pThemeNameBuffer;		// out
	size_t nThemeNameBufferSize;	// in
	unsigned long ulThemeKey;		// out, plugin defines value
	bool bAutoEnableEditing;		// out
	size_t nPenWidth;				// out
	int nIconIndex;                 // out
	bool bForceLayerOnTop;          // out
	wchar_t *pInfoTipBuffer;		// out
	size_t nInfoTipBufferSize;		// in
	bool bDrawInfoTipOnMap;			// out
	bool bFrameInfoTipOnMap;		// out
	PFNRAISEEVENTCALLBACK pfnRECB;	// in
} LoadShapeInfo;

bool __cdecl
	fnLoadShapes
		(
			LoadShapeInfo *pInfo
		);

bool __cdecl
	fnSupportsType
		(
			ShapeType t
		);

size_t __cdecl
	fnThemeCount
		(
			ShapeType t
		);

size_t __cdecl
	fnVersion
		(
			void // Current version is LOADER_PLUGIN_INTERFACE_VERSION
		);

/***************************************************************************
 *
 * The following section contains functions that are optionally implemented.
 *
 * These functions only needs to be implemented if attribute data are
 * supplied with the shape data.
 *
 * Different shape types may have different sets of attributes.
 *
 ***************************************************************************/

size_t __cdecl
	fnGetAttributeCount
		(
			unsigned long ulThemeKey
		);

const char* __cdecl
	fnGetAttributeName
		(
			unsigned long ulThemeKey,
			size_t nCol
		);

// Attribute types
//   Any of the types kan be combined with the AT_PRIMARYKEY flag
//   to indicate that the attribute is part of a primary key.
#define AT_STRING     0
#define AT_INT        1
#define AT_DOUBLE     2
#define AT_DATE       3
#define AT_BOOL       4
#define AT_PRIMARYKEY 0x80000000

unsigned long __cdecl
	fnGetAttributeType
		(
			unsigned long ulThemeKey,
			size_t nCol
		);

const char* __cdecl
	fnGetAttribute
		(
			unsigned long ulThemeKey,
			CShapeRecord *pShape,
			size_t nCol
		);

const wchar_t* __cdecl
	fnGetInfo
		(
			void
		);

bool __cdecl
	fnPreLoadShapes
		(
			void
		);

void __cdecl
	fnSettings
		(
			void
		);

void __cdecl
	fnRemoveShapes
		(
			unsigned long ulThemeKey
		);


/***************************************************************************
 *
 * The following section contains functions that are optionally implemented.
 *
 * These functions needs to be implemented if the loader plugin should
 * support serializiation.
 *
 ***************************************************************************/

enum SerializingState
{
	SS_QueryBufferSize = 0,
	SS_Store = 1,
	SS_Load = 2
};

typedef struct tagSerializeInfo
{
	SerializingState state;			// in, storing or loading
	unsigned long *pulThemeKey;		// in/out, key to this theme
	void* pStorage;					// in/out, data area to write to
	long  nStorageSize;				// in, length of pStorage
	PFNADDGRAPHICCALLBACK pfnAGCB;	// in, callback for loading
	unsigned long ulSessionData;	// in, this parameter should go into (*pfnAGCB)()
} SerializeInfo;


// If state == SS_QueryBufferSize
//   pulThemeKey is an in parameter
//   Return the number of bytes needed to serialize (store) this theme.
//   Return -1 if it's not possible to serialize.
// If state == SS_Store
//   pulThemeKey is an in parameter
//   Write the data needed to reconstruct this theme to pStorage.
//   nStorageSize will be the amount returned from the call where state == SS_QueryBufferSize
//   Return value ignored.
// If state == SS_Load
//   Parse pStorage
//   pulThemeKey is an out parameter
//   Load shapes and send them back through pfnCB
//   Return nonzero on success and zero on failure.

long __cdecl
	fnSerialize
		(
			SerializeInfo *pInfo
		);


/***************************************************************************
 *
 * The following section contains functions that are optionally implemented.
 *
 * These functions needs to be implemented if the loader plugin wants to
 * continously update data or respond to various events.
 *
 ***************************************************************************/

// Events are triggered from ShapeUp main executable.
typedef struct tagEventInfoHeader
{
	unsigned long ulThemeKey;
	unsigned long ulEventID;
} EventInfoHeader;

typedef struct tagEventInfoSelectionStateChanged
{
	EventInfoHeader hdr;
	unsigned long bNoNewGUI;              // in, if true, advises not to open any new dialogs.
	unsigned long nSelectionCount;        // in, number of currently selected shapes
	PFNGETSELECTEDSHAPECALLBACK pfnGSSCB; // in, call this to get the selected shapes
	unsigned long ulSessionData;          // in, this parameter should go into the callbacks
} EventInfoSelectionStateChanged;

// Event IDs
#define EID_SELECTIONSTATECHANGED 0

// fnHandleEvent Return codes:
#define HERC_NOMOREEVENTS 0
#define HERC_MOREEVENTS   1
#define HERC_DATACHANGED  2
// All other return codes are reserved for future use.
long __cdecl
	fnHandleEvent
		(
			EventInfoHeader *pInfoHdr
		);



// Custom events are triggered from within the plugin itself.
typedef struct tagCustomEventInfo
{
	unsigned long ulThemeKey;			// in
	PFNADDGRAPHICCALLBACK pfnAGCB;		// in, call this to add new shapes
	PFNREMOVEGRAPHICCALLBACK pfnRGCB;	// in, call this to remove a shape
	unsigned long ulSessionData;		// in, this parameter should go into the callbacks
	unsigned long ulEventHint;			// in, user defined value
	PFNSELECTGRAPHICCALLBACK pfnSGCB;	// in, call this to select a shape. NULL pShape to deselect all.
} CustomEventInfo;

// Return 1 if you want to continue getting custom events.
// Return 0 otherwise.
// All other return codes are reserved for future use.
long __cdecl
	fnHandleCustomEvent
		(
			CustomEventInfo *pInfo
		);


// Custom events can also be triggered from a custom menu.
// This menu is described as a vector of CustomMenuItems
// The first item should contain the name of the menu, and
// ulEventHint should contain the number of items, including
// the menu title item. A NULL or empty pszName will render
// a menu separator.
// The pszDescription parameter will be showed in the status
// bar when hovering the mouse over the item.
// An NULL return value is valid.
// The ulEventHint parameter is passed to the
// fnHandleCustomEvent function when the user selects this
// item.
typedef struct tagCustomMenuItem
{
	const wchar_t *pszName;
	unsigned long  ulEventHint;
	unsigned long  bActive;
	const wchar_t *pszDescription;
} CustomMenuItem;

CustomMenuItem* __cdecl
	fnGetCustomMenu
		(
			unsigned long ulThemeKey
		);

#endif /* _LDRDEF_H */
