#include "StdAfx.h"
#include ".\[!output SAFE_PROJECT_NAME]loader.h"
#include ".\[!output SAFE_PROJECT_NAME]theme.h"
[!if SETTINGS_DIALOG]
#include ".\settingsdialog.h"
[!endif]
[!if LOAD_POINT]
#include ".\shapepointcreator.h"
[!endif]
[!if LOAD_POLYLINE]
#include ".\shapearccreator.h"
[!endif]
[!if LOAD_POLYGON]
#include ".\shapepolygoncreator.h"
[!endif]
[!if SOCKETS]
#include <afxinet.h>
[!endif]

std::wstring C[!output SAFE_PROJECT_NAME]Loader::g_strErr;
[!if LOAD_POINT]
size_t C[!output SAFE_PROJECT_NAME]Loader::g_nNumberOfPointLayers = 0;
[!endif]
[!if LOAD_POLYLINE]
size_t C[!output SAFE_PROJECT_NAME]Loader::g_nNumberOfPolylineLayers = 0;
[!endif]
[!if LOAD_POLYGON]
size_t C[!output SAFE_PROJECT_NAME]Loader::g_nNumberOfPolygonLayers = 0;
[!endif]
[!if FILE_SUPPORT]
std::string C[!output SAFE_PROJECT_NAME]Loader::g_strFileName;
[!endif]


//*******************************************************************************
// Name : C[!output SAFE_PROJECT_NAME]Loader
// Desc : Default constructor
//*******************************************************************************
C[!output SAFE_PROJECT_NAME]Loader::C[!output SAFE_PROJECT_NAME]Loader(void)
{
}


//*******************************************************************************
// Name : ~C[!output SAFE_PROJECT_NAME]Loader
// Desc : Destructor
//*******************************************************************************
C[!output SAFE_PROJECT_NAME]Loader::~C[!output SAFE_PROJECT_NAME]Loader(void)
{
}


//*******************************************************************************
// Name : GetName
// Desc : Return the name that will be displayed in ShapeUp menues.
//*******************************************************************************
const wchar_t* C[!output SAFE_PROJECT_NAME]Loader::GetName()
{
	return L"Friendly name of [!output PROJECT_NAME]";
}


//*******************************************************************************
// Name : GetLastErrMsg
// Desc : Return last error message
//*******************************************************************************
const wchar_t* C[!output SAFE_PROJECT_NAME]Loader::GetLastErrMsg()
{
	return g_strErr.c_str();
}


//*******************************************************************************
// Name : GetInfo
// Desc : Return a description of this loader. This is accessible from
//        Tools | Options | Plugins | Loaders | Info button.
//*******************************************************************************
const wchar_t* C[!output SAFE_PROJECT_NAME]Loader::GetInfo()
{
	return	L"Loader plugin [!output PROJECT_NAME] version 1.0.\n"
			L"Copyright 2005.";
}


//*******************************************************************************
// Name : SupportsType
// Desc : Return the types of layers supported
//*******************************************************************************
bool C[!output SAFE_PROJECT_NAME]Loader::SupportsType(ShapeType t)
{
[!if LOAD_POINT]
	if (t == Shape_Point)
		return true;

[!endif]
[!if LOAD_POLYLINE]
	if (t == Shape_Arc)
		return true;

[!endif]
[!if LOAD_POLYGON]
	if (t == Shape_Polygon)
		return true;

[!endif]
	// We do not support this type
	return false;
}


//*******************************************************************************
// Name : GetThemeCount
// Desc : Returns the number of themes available for each type of shape this
//        loader supports.
//*******************************************************************************
size_t C[!output SAFE_PROJECT_NAME]Loader::GetThemeCount(ShapeType t)
{
[!if LOAD_POINT]
	if (t == Shape_Point)
		return g_nNumberOfPointLayers;

[!endif]
[!if LOAD_POLYLINE]
	if (t == Shape_Arc)
		return g_nNumberOfPolylineLayers;

[!endif]
[!if LOAD_POLYGON]
	if (t == Shape_Polygon)
		return g_nNumberOfPolygonLayers;

[!endif]
	// We should not end up here
	return 0;
}


//*******************************************************************************
// Name : OnPreLoadShapes
// Desc : This function gets called once at the start of this session.
//*******************************************************************************
bool C[!output SAFE_PROJECT_NAME]Loader::OnPreLoadShapes()
{
	// TODO: This is a good place to examine your data source to see what's
	//       available, and then use the result in SupportsType() and
	//       GetThemeCount() functions.
	//
	//       If you want the user to make any loading selections you don't
	//       want in a general settings dialog, you can display a dialog box
	//       from here. E.g. if loading data from a file, a file dialog
	//       could be placed here.

[!if FILE_SUPPORT]
	char szFilter[] = "MyType Files (*.my)|*.my|All Files (*.*)|*.*||";
	CFileDialog dlg(TRUE, 0, 0, OFN_HIDEREADONLY | OFN_EXPLORER, szFilter);
	if (dlg.DoModal() != IDOK)
	{
		// User pressed cancel
		return false;
	}

	g_strFileName = dlg.GetPathName();

[!endif]
[!if SOCKETS]
	try
	{
		// This try/catch was added because windows sockets was selected
		// in ShapeUp Loader Wizard.
		// Notice: This method of loading internet files may use locally
		//         cached files.
		CInternetSession session;
		CStdioFile *pFile = NULL;

		// TODO: Specify URL
//		pFile = session.OpenURL("http://www.mydomain.tld/filename.ext");

		if (pFile != NULL)
		{
			// TODO: Read data

			// Read data in text mode
//			CString strRes;
//			while (pFile->ReadString(strRes))
//				g_strData += strRes + '\n';

			// g_strData now contains the file data

			delete pFile;
		}
	}
	catch (CInternetException *ex)
	{
		ex->ReportError();
		ex->Delete();
		return false;
	}

[!endif]
[!if LOAD_POINT]
	g_nNumberOfPointLayers = 1;
[!endif]
[!if LOAD_POLYLINE]
	g_nNumberOfPolylineLayers = 1;
[!endif]
[!if LOAD_POLYGON]
	g_nNumberOfPolygonLayers = 1;
[!endif]


	// Returning false will cancel the loading process
	return true;
}


[!if SETTINGS_DIALOG]
//*******************************************************************************
// Name : OnSettings
// Desc : 
//*******************************************************************************
void C[!output SAFE_PROJECT_NAME]Loader::OnSettings()
{
	CSettingsDialog dlg;

	// TODO: Populate dlg with data from registry

	if (dlg.DoModal())
	{
		// TODO: Collect data from dlg and store in registry
	}
}


[!endif]
//*******************************************************************************
// Name : OnLoadShapes
// Desc : 
//*******************************************************************************
bool C[!output SAFE_PROJECT_NAME]Loader::OnLoadShapes(LoadShapeInfo *pInfo, C[!output SAFE_PROJECT_NAME]Theme &theme)
{
	// TODO: Read/create shape objects and send them to ShapeUp

	size_t nRecordNumber = 0;
	size_t nExtraDataBytes = 0;

	bool bMoreShapesAvailable = true;

	while (bMoreShapesAvailable)
	{
		CShapeRecord *pShape = NULL;
		switch (pInfo->type)
		{
[!if LOAD_POINT]
		case Shape_Point:
			{
				// TODO: If loading more than one point layer, check pInfo->nThemeIndex
				//       to figure out which one is currently loading, or use a member
				//       or class variable to keep track of it.

				// Set a default theme name
				lstrcpynW(pInfo->pThemeNameBuffer, L"Points", static_cast<int>(pInfo->nThemeNameBufferSize));
				pInfo->pThemeNameBuffer[pInfo->nThemeNameBufferSize - 1] = L'\0';

				// TODO: Get the point data from somewhere:
				double x = 0.0;
				double y = 0.0;

[!if SUPPORTS_ATTRIBUTES]
				// Calculate space needed for attribute data
				nExtraDataBytes = sizeof(C[!output SAFE_PROJECT_NAME]Theme::PointTheme1Attributes);

[!endif]
				// Use a record creator to construct the shape
				CShapePointCreator creator(nRecordNumber);

				// Create space for the shape
[!if SUPPORTS_ATTRIBUTES]
				pShape = CreateShape(creator.GetAllocSizeBytes() + nExtraDataBytes);
[!else]
				pShape = CreateShape(creator.GetAllocSizeBytes());
[!endif]

				if (pShape == NULL)
				{
					g_strErr = L"Out of memory error in loader.";
					return false; // Mem allocation error
				}

				// Set the shapes coordinates
				creator.Write(pShape, x, y);

[!if SUPPORTS_ATTRIBUTES]
				// TODO: Write attribute data
				C[!output SAFE_PROJECT_NAME]Theme::PointTheme1Attributes *pAttr = reinterpret_cast<C[!output SAFE_PROJECT_NAME]Theme::PointTheme1Attributes*>(pShape->GetExtraDataPtr());
				strcpy(pAttr->value1, "val1");
				strcpy(pAttr->value2, "val2");

[!endif]
				// Set theme type
				theme.SetThemeId(C[!output SAFE_PROJECT_NAME]Theme::PointTheme_1);

				// If we have no more data:
				bMoreShapesAvailable = false;
			}
			break;
[!endif]
[!if LOAD_POLYLINE]
		case Shape_Arc:
			{
				// TODO: If loading more than one polyline layer, check pInfo->nThemeIndex
				//       to figure out which one is currently loading, or use a member
				//       or class variable to keep track of it.

				// Set a default theme name
				lstrcpynW(pInfo->pThemeNameBuffer, L"Polylines", static_cast<int>(pInfo->nThemeNameBufferSize));
				pInfo->pThemeNameBuffer[pInfo->nThemeNameBufferSize - 1] = L'\0';

				// TODO: Get the point data from somewhere:
				double x1 = 0.0;
				double y1 = 0.0;
				double x2 = 1.0;
				double y2 = 1.0;

[!if SUPPORTS_ATTRIBUTES]
				// Calculate space needed for attribute data
				nExtraDataBytes = sizeof(C[!output SAFE_PROJECT_NAME]Theme::PolylineTheme1Attributes);

[!endif]
				// How many part should the arc contain
				size_t nPartCount = 1; // Almost always 1

				// TODO: Calculate how many break-points are needed
				//       (The sum of all break-points for all parts)
				size_t nTotalPointCount = 2;

				// Use a record creator to construct the shape
				CShapeArcCreator creator(nPartCount, nTotalPointCount, nRecordNumber);

				// Create space for the shape
[!if SUPPORTS_ATTRIBUTES]
				pShape = CreateShape(creator.GetAllocSizeBytes() + nExtraDataBytes);
[!else]
				pShape = CreateShape(creator.GetAllocSizeBytes());
[!endif]

				if (pShape == NULL)
				{
					g_strErr = L"Out of memory error in loader.";
					return false; // Mem allocation error
				}

				// Set the shapes data
				creator.WriteHeader(pShape);
				for (size_t nPart = 0; nPart < nPartCount; ++nPart)
				{
					// Set coords
					creator.WritePoint(x1, y1);
					creator.WritePoint(x2, y2);
					creator.WriteNewPart();
				}

[!if SUPPORTS_ATTRIBUTES]
				// TODO: Write attribute data
				C[!output SAFE_PROJECT_NAME]Theme::PolylineTheme1Attributes *pAttr = reinterpret_cast<C[!output SAFE_PROJECT_NAME]Theme::PolylineTheme1Attributes*>(pShape->GetExtraDataPtr());
				strcpy(pAttr->value1, "val1");
				strcpy(pAttr->value2, "val2");

[!endif]
				// Set theme type
				theme.SetThemeId(C[!output SAFE_PROJECT_NAME]Theme::PolylineTheme_1);

				// If we have no more data:
				bMoreShapesAvailable = false;
			}
			break;
[!endif]
[!if LOAD_POLYGON]
		case Shape_Polygon:
			{
				// TODO: If loading more than one polygon layer, check pInfo->nThemeIndex
				//       to figure out which one is currently loading, or use a member
				//       or class variable to keep track of it.

				// Set a default theme name
				lstrcpynW(pInfo->pThemeNameBuffer, L"Polygons", static_cast<int>(pInfo->nThemeNameBufferSize));
				pInfo->pThemeNameBuffer[pInfo->nThemeNameBufferSize - 1] = L'\0';

				// TODO: Get the point data from somewhere:
				double x1 = 0.0;
				double y1 = 0.0;
				double x2 = 1.0;
				double y2 = 1.0;
				double x3 = 1.0;
				double y3 = 0.0;

[!if SUPPORTS_ATTRIBUTES]
				// Calculate space needed for attribute data
				nExtraDataBytes = sizeof(C[!output SAFE_PROJECT_NAME]Theme::PolygonTheme1Attributes);

[!endif]
				// How many part should the polygon contain
				size_t nPartCount = 1; // Almost always 1

				// TODO: Calculate how many break-points are needed
				//       (The sum of all break-points for all parts)
				// NOTE: A polygon part contains at least 4 coordinates.
				//       The first and the last must be the same.
				size_t nTotalPointCount = 4;

				// Use a record creator to construct the shape
				CShapePolygonCreator creator(nPartCount, nTotalPointCount, nRecordNumber);

				// Create space for the shape
[!if SUPPORTS_ATTRIBUTES]
				pShape = CreateShape(creator.GetAllocSizeBytes() + nExtraDataBytes);
[!else]
				pShape = CreateShape(creator.GetAllocSizeBytes());
[!endif]

				if (pShape == NULL)
				{
					g_strErr = L"Out of memory error in loader.";
					return false; // Mem allocation error
				}

				// Set the shapes data
				creator.WriteHeader(pShape);
				for (size_t nPart = 0; nPart < nPartCount; ++nPart)
				{
					// Set coords
					creator.WritePoint(x1, y1);
					creator.WritePoint(x2, y2);
					creator.WritePoint(x3, y3);
					// Close the part by adding the first coordinate again. (Shape polygon constraint)
					creator.WritePoint(x1, y1);
					creator.WriteNewPart();
				}

[!if SUPPORTS_ATTRIBUTES]
				// TODO: Write attribute data
				C[!output SAFE_PROJECT_NAME]Theme::PolygonTheme1Attributes *pAttr = reinterpret_cast<C[!output SAFE_PROJECT_NAME]Theme::PolygonTheme1Attributes*>(pShape->GetExtraDataPtr());
				strcpy(pAttr->value1, "val1");
				strcpy(pAttr->value2, "val2");

[!endif]
				// Set theme type
				theme.SetThemeId(C[!output SAFE_PROJECT_NAME]Theme::PolygonTheme_1);

				// If we have no more data:
				bMoreShapesAvailable = false;
			}
			break;
[!endif]
		default:
			// Ending up here implies a coding error in SupportsType()
			ASSERT(FALSE);
			return false;
		}

		// Notify ShapeUp of this new shape
		if (pInfo->pfnAGCB(pInfo->ulSessionData, pShape, nExtraDataBytes))
		{
			// If accepted, store it in the theme and increase record number
			theme.AddShape(pShape);
			++nRecordNumber;
		}
		else
		{
			// If it was rejected, delete it
			DeleteShape(pShape);
		}
	}

	// Returning false will abort the loading process
	return true;
}


//*******************************************************************************
// Name : CreateShape
// Desc : This function creates a CShapeRecord of the required space.
//*******************************************************************************
CShapeRecord* C[!output SAFE_PROJECT_NAME]Loader::CreateShape(size_t nBytesRequired)
{
	unsigned char *pBytes = new unsigned char[nBytesRequired];
	return reinterpret_cast<CShapeRecord*>(pBytes);
}


//*******************************************************************************
// Name : DeleteShape
// Desc : Delete the specified shape. The shape must have been created using
//        the function CreateShape() above.
//*******************************************************************************
void C[!output SAFE_PROJECT_NAME]Loader::DeleteShape(CShapeRecord *pShape)
{
	delete reinterpret_cast<unsigned char*>(pShape);
}


[!if CUSTOM_MENU_SUPPORT]
//*******************************************************************************
// Custom events
#define CE_SAMPLE_MENU_ITEM        1


//*******************************************************************************
// Name : GetCustomMenu
// Desc : 
//*******************************************************************************
CustomMenuItem* C[!output SAFE_PROJECT_NAME]Loader::GetCustomMenu(C[!output SAFE_PROJECT_NAME]Theme &theme)
{
	static CustomMenuItem menu1[] =
	{
		// First row contains menu name & number of items (including menu name & separators)
		{ L"[!output PROJECT_NAME]", 2, TRUE, NULL },
		{ L"&Sample menu item", CE_SAMPLE_MENU_ITEM,
			TRUE, L"Opens the route creator dialog" },
		//{ NULL, 0, FALSE, NULL }, // This is a menu separator item
	};

	switch (theme.GetThemeId())
	{
[!if LOAD_POINT]
	case C[!output SAFE_PROJECT_NAME]Theme::PointTheme_1:
		return menu1;
[!endif]
[!if LOAD_POLYLINE]
	case C[!output SAFE_PROJECT_NAME]Theme::PolylineTheme_1:
		return menu1;
[!endif]
[!if LOAD_POLYGON]
	case C[!output SAFE_PROJECT_NAME]Theme::PolygonTheme_1:
		return menu1;
[!endif]
	default:
		break;
	}

	return NULL;
}


//*******************************************************************************
// Name : OnCustomEvent
// Desc : 
//*******************************************************************************
long C[!output SAFE_PROJECT_NAME]Loader::OnCustomEvent(C[!output SAFE_PROJECT_NAME]Theme &theme, CustomEventInfo *pInfo)
{
	// You are not allowed to use AFX_MANAGE_STATE(AfxGetStaticModuleState())
	// if you are to use any of the callbacks. If you need it because of
	// resources, use it inside a block not using the callbacks.

	switch (pInfo->ulEventHint)
	{
	case CE_SAMPLE_MENU_ITEM:
		{
			// This block will not use the callbacks
			AFX_MANAGE_STATE(AfxGetStaticModuleState());

			// TODO: Do something useful
			AfxMessageBox("Sample menu item response!", MB_OK | MB_ICONINFORMATION, 0);
		}
		break;
	default:
		break;
	}
	return 1;
}


[!endif]
