#include "stdafx.h"
#include "shapeobjects.h"


//*******************************************************************************
// Name : RecordSize
// Desc : 
//*******************************************************************************
size_t CShapeRecord::RecordSize() const
{
	switch (GetShapeType())
	{
	case Shape_Point:
	case Shape_PointZ:
		return reinterpret_cast<const CShapePointRecord*>(this)->RecordSize();
	case Shape_Arc:
	case Shape_PolyLineZ:
		return reinterpret_cast<const CShapeArcRecord*>(this)->RecordSize();
	case Shape_Polygon:
		return reinterpret_cast<const CShapePolygonRecord*>(this)->RecordSize();
	default:
		ASSERT(FALSE);
		break;
	}

	return 0;
}


//*******************************************************************************
// Name : DataSize
// Desc : 
//*******************************************************************************
size_t CShapeRecord::DataSize() const
{
	switch (GetShapeType())
	{
	case Shape_Point:
	case Shape_PointZ:
		return reinterpret_cast<const CShapePointRecord*>(this)->DataSize();
	case Shape_Arc:
	case Shape_PolyLineZ:
		return reinterpret_cast<const CShapeArcRecord*>(this)->DataSize();
	case Shape_Polygon:
		return reinterpret_cast<const CShapePolygonRecord*>(this)->DataSize();
	default:
		ASSERT(FALSE);
		break;
	}

	return 0;
}


//*******************************************************************************
// Name : GetXMin
// Desc : 
//*******************************************************************************
double CShapeRecord::GetXMin() const
{
	switch (GetShapeType())
	{
	case Shape_Point:
	case Shape_PointZ:
		return reinterpret_cast<const CShapePointRecord*>(this)->GetX();
	case Shape_Arc:
	case Shape_PolyLineZ:
		return reinterpret_cast<const CShapeArcRecord*>(this)->GetXMin();
	case Shape_Polygon:
		return reinterpret_cast<const CShapePolygonRecord*>(this)->GetXMin();
	default:
		ASSERT(FALSE);
		break;
	}

	return 0.0;
}


//*******************************************************************************
// Name : GetYMin
// Desc : 
//*******************************************************************************
double CShapeRecord::GetYMin() const
{
	switch (GetShapeType())
	{
	case Shape_Point:
	case Shape_PointZ:
		return reinterpret_cast<const CShapePointRecord*>(this)->GetY();
	case Shape_Arc:
	case Shape_PolyLineZ:
		return reinterpret_cast<const CShapeArcRecord*>(this)->GetYMin();
	case Shape_Polygon:
		return reinterpret_cast<const CShapePolygonRecord*>(this)->GetYMin();
	default:
		ASSERT(FALSE);
		break;
	}

	return 0.0;
}


//*******************************************************************************
// Name : GetXMax
// Desc : 
//*******************************************************************************
double CShapeRecord::GetXMax() const
{
	switch (GetShapeType())
	{
	case Shape_Point:
	case Shape_PointZ:
		return reinterpret_cast<const CShapePointRecord*>(this)->GetX();
	case Shape_Arc:
	case Shape_PolyLineZ:
		return reinterpret_cast<const CShapeArcRecord*>(this)->GetXMax();
	case Shape_Polygon:
		return reinterpret_cast<const CShapePolygonRecord*>(this)->GetXMax();
	default:
		ASSERT(FALSE);
		break;
	}

	return 0.0;
}


//*******************************************************************************
// Name : GetYMax
// Desc : 
//*******************************************************************************
double CShapeRecord::GetYMax() const
{
	switch (GetShapeType())
	{
	case Shape_Point:
	case Shape_PointZ:
		return reinterpret_cast<const CShapePointRecord*>(this)->GetY();
	case Shape_Arc:
	case Shape_PolyLineZ:
		return reinterpret_cast<const CShapeArcRecord*>(this)->GetYMax();
	case Shape_Polygon:
		return reinterpret_cast<const CShapePolygonRecord*>(this)->GetYMax();
	default:
		ASSERT(FALSE);
		break;
	}

	return 0.0;
}


//*******************************************************************************
// Name : GetPartCount
// Desc : 
//*******************************************************************************
size_t CShapeRecord::GetPartCount()   const
{
	switch (GetShapeType())
	{
	case Shape_Point:
	case Shape_PointZ:
		return reinterpret_cast<const CShapePointRecord*>(this)->GetPartCount();
	case Shape_Arc:
	case Shape_PolyLineZ:
		return reinterpret_cast<const CShapeArcRecord*>(this)->GetPartCount();
	case Shape_Polygon:
		return reinterpret_cast<const CShapePolygonRecord*>(this)->GetPartCount();
	default:
		ASSERT(FALSE);
		break;
	}

	return 0;
}


//*******************************************************************************
// Name : GetPointCount
// Desc : 
//*******************************************************************************
size_t CShapeRecord::GetPointCount() const
{
	switch (GetShapeType())
	{
	case Shape_Point:
	case Shape_PointZ:
		return reinterpret_cast<const CShapePointRecord*>(this)->GetPointCount();
	case Shape_Arc:
	case Shape_PolyLineZ:
		return reinterpret_cast<const CShapeArcRecord*>(this)->GetPointCount();
	case Shape_Polygon:
		return reinterpret_cast<const CShapePolygonRecord*>(this)->GetPointCount();
	default:
		ASSERT(FALSE);
		break;
	}

	return 0;
}


//*******************************************************************************
// Name : GetPoint
// Desc : 
//*******************************************************************************
const CShapeRecord::XYPair& CShapeRecord::GetPoint(size_t idx) const
{
	switch (GetShapeType())
	{
	case Shape_Point:
	case Shape_PointZ:
		return *reinterpret_cast<const CShapePointRecord*>(this)->GetPoint();
	case Shape_Arc:
	case Shape_PolyLineZ:
		return reinterpret_cast<const CShapeArcRecord*>(this)->GetPoint(idx);
	case Shape_Polygon:
		return reinterpret_cast<const CShapePolygonRecord*>(this)->GetPoint(idx);
	default:
		ASSERT(FALSE);
		break;
	}

	static XYPair xy = { 0., 0. };
	return xy;
}


//*******************************************************************************
// Name : GetPart
// Desc : 
//*******************************************************************************
size_t CShapeRecord::GetPart(size_t idx) const
{
	switch (GetShapeType())
	{
	case Shape_Point:
	case Shape_PointZ:
		ASSERT(idx == 0);
		return 0;
	case Shape_Arc:
	case Shape_PolyLineZ:
		return reinterpret_cast<const CShapeArcRecord*>(this)->GetPart(idx);
	case Shape_Polygon:
		return reinterpret_cast<const CShapePolygonRecord*>(this)->GetPart(idx);
	default:
		ASSERT(FALSE);
		break;
	}

	return 0;
}


//*******************************************************************************
// Name : GetZ
// Desc : 
//*******************************************************************************
double CShapeRecord::GetZ(size_t idx) const
{
	switch (GetShapeType())
	{
	case Shape_Point:
	case Shape_PointZ:
		return reinterpret_cast<const CShapePointRecord*>(this)->GetZ();
	case Shape_Arc:
	case Shape_PolyLineZ:
		return reinterpret_cast<const CShapeArcRecord*>(this)->GetZ(idx);
	case Shape_Polygon:
		return reinterpret_cast<const CShapePolygonRecord*>(this)->GetZ(idx);
	default:
		ASSERT(FALSE);
		break;
	}

	return 0.;
}


//*******************************************************************************
// Name : GetM
// Desc : 
//*******************************************************************************
double CShapeRecord::GetM(size_t idx) const
{
	switch (GetShapeType())
	{
	case Shape_Point:
	case Shape_PointZ:
		return reinterpret_cast<const CShapePointRecord*>(this)->GetM();
	case Shape_Arc:
	case Shape_PolyLineZ:
		return reinterpret_cast<const CShapeArcRecord*>(this)->GetM(idx);
	case Shape_Polygon:
		return reinterpret_cast<const CShapePolygonRecord*>(this)->GetM(idx);
	default:
		ASSERT(FALSE);
		break;
	}

	return 0.;
}


//*******************************************************************************
// Name : GetExtraDataPtr
// Desc : 
//*******************************************************************************
void* CShapeRecord::GetExtraDataPtr()
{
	switch (GetShapeType())
	{
	case Shape_Point:
	case Shape_PointZ:
		return reinterpret_cast<CShapePointRecord*>(this)->GetExtraDataPtr();
	case Shape_Arc:
	case Shape_PolyLineZ:
		return reinterpret_cast<CShapeArcRecord*>(this)->GetExtraDataPtr();
	case Shape_Polygon:
		return reinterpret_cast<CShapePolygonRecord*>(this)->GetExtraDataPtr();
	}

	return NULL;
}


/*------------------------------------------------------------------------
-- Name : CShapePointRecord::CreateRecord
-- Descr: Special Constructor
------------------------------------------------------------------------*/
CShapePointRecord* CShapePointRecord::CreateRecord(void* pData)
{
	return reinterpret_cast<CShapePointRecord*>(pData);
}


/*------------------------------------------------------------------------
-- Name : CShapeArcRecord::CreateRecord
-- Descr: Special Constructor
------------------------------------------------------------------------*/
CShapeArcRecord* CShapeArcRecord::CreateRecord(void* pData)
{
	return reinterpret_cast<CShapeArcRecord*>(pData);
}

